@@ -154,8 +154,10 @@ struct TESSLINE {
            botright.y <= pt.y && pt.y <= topleft.y;
   }
 
+  #ifndef GRAPHICS_DISABLED
   void plot(ScrollView* window, ScrollView::Color color,
             ScrollView::Color child_color);
+  #endif  // GRAPHICS_DISABLED
 
   int BBArea() const {
     return (botright.x - topleft.x) * (topleft.y - botright.y);
@@ -210,8 +212,10 @@ struct TBLOB {
 
   TBOX bounding_box() const;
 
+  #ifndef GRAPHICS_DISABLED
   void plot(ScrollView* window, ScrollView::Color color,
             ScrollView::Color child_color);
+  #endif  // GRAPHICS_DISABLED
 
   int BBArea() const {
     int total_area = 0;
