@@ -65,7 +65,9 @@ class DLLSYM POLY_BLOCK {
 
   void plot(ScrollView* window, inT32 num);
 
+  #ifndef GRAPHICS_DISABLED
   void fill(ScrollView* window, ScrollView::Color colour);
+  #endif  // GRAPHICS_DISABLED
 
   // Returns true if other is inside this.
   bool contains(POLY_BLOCK *other);
@@ -78,10 +80,11 @@ class DLLSYM POLY_BLOCK {
   // test_pt outside this.
   inT16 winding_number(const ICOORD &test_pt);
 
+  #ifndef GRAPHICS_DISABLED
   // Static utility functions to handle the PolyBlockType.
-
   // Returns a color to draw the given type.
   static ScrollView::Color ColorForPolyBlockType(PolyBlockType type);
+  #endif  // GRAPHICS_DISABLED
 
  private:
   ICOORDELT_LIST vertices;     // vertices
