@@ -24,7 +24,7 @@
 #include "genericvector.h"
 #include "unicharset.h"
 
-ELISTIZE (BLOB_CHOICE) CLISTIZE (BLOB_CHOICE_LIST) CLISTIZE (WERD_CHOICE)
+ELISTIZE (BLOB_CHOICE) CLISTIZE (BLOB_CHOICE_LIST) CLISTIZE (WERD_CHOICE);
 
 const float WERD_CHOICE::kBadRating = 100000.0;
 
