@@ -19,14 +19,14 @@
 #ifndef   KDTREE_H
 #define   KDTREE_H
 
-/**----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
           Include Files and Type Defines
-----------------------------------------------------------------------------**/
-#include "general.h"
+-----------------------------------------------------------------------------*/
+#include "host.h"
 #include "cutil.h"
 #include "ocrfeatures.h"
 
-/*
+/**
 NOTE:  All circular parameters of all keys must be in the range
 
 Min <= Param < Max
@@ -36,83 +36,61 @@ MakeKDTree.  All KD routines assume that this is true and will not operate
 correctly if circular parameters outside the specified range are used.
 */
 
-typedef struct kdnode
-{
-  FLOAT32 *Key;                  /* search key */
-  char *Data;                    /* data that corresponds to key */
-  FLOAT32 BranchPoint;           /* needed to make deletes work efficiently */
-  FLOAT32 LeftBranch;            /* used to optimize search pruning */
-  FLOAT32 RightBranch;           /* used to optimize search pruning */
-  struct kdnode *Left;           /* ptrs for KD tree structure */
-  struct kdnode *Right;
-}
+struct KDNODE {
+  FLOAT32 *Key;                  /**< search key */
+  void *Data;                    /**< data that corresponds to key */
+  FLOAT32 BranchPoint;           /**< needed to make deletes work efficiently */
+  FLOAT32 LeftBranch;            /**< used to optimize search pruning */
+  FLOAT32 RightBranch;           /**< used to optimize search pruning */
+  struct KDNODE *Left;           /**< ptrs for KD tree structure */
+  struct KDNODE *Right;
+};
 
-
-KDNODE;
-
-typedef struct
-{
+struct KDTREE {
   inT16 KeySize;                 /* number of dimensions in the tree */
   KDNODE Root;                   /* Root.Left points to actual root node */
   PARAM_DESC KeyDesc[1];         /* description of each dimension */
-}
-
-
-KDTREE;
-
-typedef enum {                   /* used for walking thru KD trees */
-  preorder, postorder, endorder, leaf
-}
-
-
-VISIT;
+};
 
 /*----------------------------------------------------------------------------
             Macros
 -----------------------------------------------------------------------------*/
 #define RootOf(T)   ((T)->Root.Left->Data)
 
-/**----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
           Public Function Prototypes
-----------------------------------------------------------------------------**/
-KDTREE *MakeKDTree (inT16 KeySize, PARAM_DESC KeyDesc[]);
+-----------------------------------------------------------------------------*/
+KDTREE *MakeKDTree(inT16 KeySize, const PARAM_DESC KeyDesc[]);
 
 void KDStore(KDTREE *Tree, FLOAT32 *Key, void *Data);
 
-void KDDelete (KDTREE * Tree, FLOAT32 Key[], void *Data);
+void KDDelete(KDTREE * Tree, FLOAT32 Key[], void *Data);
 
-int KDNearestNeighborSearch (KDTREE * Tree,
-FLOAT32 Query[],
-int QuerySize,
-FLOAT32 MaxDistance,
-void *NBuffer, FLOAT32 DBuffer[]);
+void KDNearestNeighborSearch(
+    KDTREE *Tree, FLOAT32 Query[], int QuerySize, FLOAT32 MaxDistance,
+    int *NumberOfResults, void **NBuffer, FLOAT32 DBuffer[]);
 
-void KDWalk(KDTREE *Tree, void_proc Action);
+void KDWalk(KDTREE *Tree, void_proc Action, void *context);
 
 void FreeKDTree(KDTREE *Tree);
 
-/**----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
           Private Function Prototypes
-----------------------------------------------------------------------------**/
-int Equal (FLOAT32 Key1[], FLOAT32 Key2[]);
-
-KDNODE *MakeKDNode (FLOAT32 Key[], char *Data, int Index);
+-----------------------------------------------------------------------------*/
+KDNODE *MakeKDNode(KDTREE *tree, FLOAT32 Key[], void *Data, int Index);
 
 void FreeKDNode(KDNODE *Node);
 
-void Search(int Level, KDNODE *SubTree);
+FLOAT32 DistanceSquared(int k, PARAM_DESC *dim, FLOAT32 p1[], FLOAT32 p2[]);
 
-FLOAT32 ComputeDistance (register int N,
-register PARAM_DESC Dim[],
-register FLOAT32 p1[], register FLOAT32 p2[]);
+FLOAT32 ComputeDistance(int k, PARAM_DESC *dim, FLOAT32 p1[], FLOAT32 p2[]);
 
-void FindMaxDistance();
+int QueryInSearch(KDTREE *tree);
 
-int QueryIntersectsSearch();
+void Walk(KDTREE *tree, void_proc action, void *context,
+          KDNODE *SubTree, inT32 Level);
 
-int QueryInSearch();
-
-void Walk(KDNODE *SubTree, inT32 Level);
+void InsertNodes(KDTREE *tree, KDNODE *nodes);
 
 void FreeSubTree(KDNODE *SubTree);
 #endif
