@@ -33,7 +33,7 @@
 #include "normfeat.h"
 #include "scanutils.h"
 #include "unicharset.h"
-#include "varable.h"
+#include "params.h"
 
 struct NORM_PROTOS
 {
@@ -46,7 +46,7 @@ struct NORM_PROTOS
 /**----------------------------------------------------------------------------
           Private Function Prototypes
 ----------------------------------------------------------------------------**/
-FLOAT32 NormEvidenceOf(register FLOAT32 NormAdj);
+double NormEvidenceOf(register double NormAdj);
 
 void PrintNormMatch(FILE *File,
                     int NumParams,
@@ -62,13 +62,16 @@ NORM_PROTOS *ReadNormProtos(FILE *File);
 /* control knobs used to control the normalization adjustment process */
 double_VAR(classify_norm_adj_midpoint, 32.0, "Norm adjust midpoint ...");
 double_VAR(classify_norm_adj_curl, 2.0, "Norm adjust curl ...");
+// Weight of width variance against height and vertical position.
+const double kWidthErrorWeighting = 0.125;
 
 /**----------------------------------------------------------------------------
               Public Code
 ----------------------------------------------------------------------------**/
 /*---------------------------------------------------------------------------*/
 namespace tesseract {
-FLOAT32 Classify::ComputeNormMatch(CLASS_ID ClassId, FEATURE Feature,
+FLOAT32 Classify::ComputeNormMatch(CLASS_ID ClassId,
+                                   const FEATURE_STRUCT& feature,
                                    BOOL8 DebugMatch) {
 /*
  **	Parameters:
@@ -94,12 +97,12 @@ FLOAT32 Classify::ComputeNormMatch(CLASS_ID ClassId, FEATURE Feature,
   /* handle requests for classification as noise */
   if (ClassId == NO_CLASS) {
     /* kludge - clean up constants and make into control knobs later */
-    Match = (Feature->Params[CharNormLength] *
-      Feature->Params[CharNormLength] * 500.0 +
-      Feature->Params[CharNormRx] *
-      Feature->Params[CharNormRx] * 8000.0 +
-      Feature->Params[CharNormRy] *
-      Feature->Params[CharNormRy] * 8000.0);
+    Match = (feature.Params[CharNormLength] *
+      feature.Params[CharNormLength] * 500.0 +
+      feature.Params[CharNormRx] *
+      feature.Params[CharNormRx] * 8000.0 +
+      feature.Params[CharNormRy] *
+      feature.Params[CharNormRy] * 8000.0);
     return (1.0 - NormEvidenceOf (Match));
   }
 
@@ -107,33 +110,48 @@ FLOAT32 Classify::ComputeNormMatch(CLASS_ID ClassId, FEATURE Feature,
   Protos = NormProtos->Protos[ClassId];
 
   if (DebugMatch) {
-    cprintf ("\nFeature = ");
-    WriteFeature(stdout, Feature);
+    tprintf("\nChar norm for class %s\n", unicharset.id_to_unichar(ClassId));
   }
 
   ProtoId = 0;
   iterate(Protos) {
     Proto = (PROTOTYPE *) first_node (Protos);
-    Delta = Feature->Params[CharNormY] - Proto->Mean[CharNormY];
+    Delta = feature.Params[CharNormY] - Proto->Mean[CharNormY];
     Match = Delta * Delta * Proto->Weight.Elliptical[CharNormY];
-    Delta = Feature->Params[CharNormRx] - Proto->Mean[CharNormRx];
+    if (DebugMatch) {
+      tprintf("YMiddle: Proto=%g, Delta=%g, Var=%g, Dist=%g\n",
+              Proto->Mean[CharNormY], Delta,
+              Proto->Weight.Elliptical[CharNormY], Match);
+    }
+    Delta = feature.Params[CharNormRx] - Proto->Mean[CharNormRx];
     Match += Delta * Delta * Proto->Weight.Elliptical[CharNormRx];
+    if (DebugMatch) {
+      tprintf("Height: Proto=%g, Delta=%g, Var=%g, Dist=%g\n",
+              Proto->Mean[CharNormRx], Delta,
+              Proto->Weight.Elliptical[CharNormRx], Match);
+    }
+    // Ry is width! See intfx.cpp.
+    Delta = feature.Params[CharNormRy] - Proto->Mean[CharNormRy];
+    if (DebugMatch) {
+      tprintf("Width: Proto=%g, Delta=%g, Var=%g\n",
+              Proto->Mean[CharNormRy], Delta,
+              Proto->Weight.Elliptical[CharNormRy]);
+    }
+    Delta = Delta * Delta * Proto->Weight.Elliptical[CharNormRy];
+    Delta *= kWidthErrorWeighting;
+    Match += Delta;
+    if (DebugMatch) {
+      tprintf("Total Dist=%g, scaled=%g, sigmoid=%g, penalty=%g\n",
+              Match, Match / classify_norm_adj_midpoint,
+              NormEvidenceOf(Match), 256 * (1 - NormEvidenceOf(Match)));
+    }
 
     if (Match < BestMatch)
       BestMatch = Match;
 
-    if (DebugMatch) {
-      cprintf ("Proto %1d = ", ProtoId);
-      WriteNFloats (stdout, NormProtos->NumParams, Proto->Mean);
-      cprintf ("      var = ");
-      WriteNFloats (stdout, NormProtos->NumParams,
-        Proto->Variance.Elliptical);
-      cprintf ("    match = ");
-      PrintNormMatch (stdout, NormProtos->NumParams, Proto, Feature);
-    }
     ProtoId++;
   }
-  return (1.0 - NormEvidenceOf (BestMatch));
+  return 1.0 - NormEvidenceOf(BestMatch);
 }                                /* ComputeNormMatch */
 
 void Classify::FreeNormProtos() {
@@ -158,7 +176,7 @@ void Classify::FreeNormProtos() {
  * normalization adjustment.  The equation that represents the transform is:
  *       1 / (1 + (NormAdj / midpoint) ^ curl)
  **********************************************************************/
-FLOAT32 NormEvidenceOf(register FLOAT32 NormAdj) {
+double NormEvidenceOf(register double NormAdj) {
   NormAdj /= classify_norm_adj_midpoint;
 
   if (classify_norm_adj_curl == 3)
@@ -166,7 +184,7 @@ FLOAT32 NormEvidenceOf(register FLOAT32 NormAdj) {
   else if (classify_norm_adj_curl == 2)
     NormAdj = NormAdj * NormAdj;
   else
-    NormAdj = pow(static_cast<double>(NormAdj), classify_norm_adj_curl);
+    NormAdj = pow (NormAdj, classify_norm_adj_curl);
   return (1.0 / (1.0 + NormAdj));
 }
 
@@ -223,7 +241,7 @@ NORM_PROTOS *Classify::ReadNormProtos(FILE *File, inT64 end_offset) {
  */
   NORM_PROTOS *NormProtos;
   int i;
-  char unichar[UNICHAR_LEN + 1];
+  char unichar[2 * UNICHAR_LEN + 1];
   UNICHAR_ID unichar_id;
   LIST Protos;
   int NumProtos;
@@ -233,7 +251,7 @@ NORM_PROTOS *Classify::ReadNormProtos(FILE *File, inT64 end_offset) {
   NormProtos->NumProtos = unicharset.size();
   NormProtos->Protos = (LIST *) Emalloc (NormProtos->NumProtos * sizeof(LIST));
   for (i = 0; i < NormProtos->NumProtos; i++)
-    NormProtos->Protos[i] = NIL;
+    NormProtos->Protos[i] = NIL_LIST;
 
   /* read file header and save in data structure */
   NormProtos->NumParams = ReadSampleSize (File);
@@ -249,8 +267,12 @@ NORM_PROTOS *Classify::ReadNormProtos(FILE *File, inT64 end_offset) {
         Protos =
             push_last (Protos, ReadPrototype (File, NormProtos->NumParams));
       NormProtos->Protos[unichar_id] = Protos;
-    } else
-      cprintf("Error: unichar %s in normproto file is not in unichar set.\n");
+    } else {
+      cprintf("Error: unichar %s in normproto file is not in unichar set.\n",
+              unichar);
+      for (i = 0; i < NumProtos; i++)
+        FreePrototype(ReadPrototype (File, NormProtos->NumParams));
+    }
     SkipNewline(File);
   }
   return (NormProtos);
