@@ -77,6 +77,7 @@ void TextlineProjection::ConstructProjection(TO_BLOCK* input_block,
 // Display the blobs in the window colored according to textline quality.
 void TextlineProjection::PlotGradedBlobs(BLOBNBOX_LIST* blobs,
                                          ScrollView* win) {
+  #ifndef GRAPHICS_DISABLED
   BLOBNBOX_IT it(blobs);
   for (it.mark_cycle_pt(); !it.cycled_list(); it.forward()) {
     BLOBNBOX* blob = it.data();
@@ -89,6 +90,7 @@ void TextlineProjection::PlotGradedBlobs(BLOBNBOX_LIST* blobs,
     win->Rectangle(box.left(), box.bottom(), box.right(), box.top());
   }
   win->Update();
+  #endif  // GRAPHICS_DISABLED
 }
 
 // Moves blobs that look like they don't sit well on a textline from the
